@@ -1,4 +1,5 @@
 import io
+import logging
 
 from datetime import datetime, timedelta
 import json
@@ -11,8 +12,9 @@ import pandas as pd
 
 
 class GSDBS:
-    def __init__(self, creadPath):
-        self.version = "0.2.5-39"
+    def __init__(self, creadPath, configstr=""):
+        self.version = "0.2.6-7"
+        self._logger = logging.getLogger(__name__)
         self.credentials = None
         self.accessToken = None
         self.dtablename = None
@@ -21,7 +23,9 @@ class GSDBS:
         self.data = pd.DataFrame()
         self.statement = None
         self.creadPath = creadPath
+        self.configstr = configstr
         self.readCredentials()
+        self.cookiejar = {}
         self.getTokenFromApi()
         self.CGSResponseCode = {}
         self.setupCGSResponseCodes()
@@ -34,56 +38,53 @@ class GSDBS:
 
     def readCredentials(self):
         try:
-            print(f'GS-DBMS Python client version: {self.version}')
-            f = open(self.creadPath + "/gscred.yml", "r")
-            cred = f.read()
-            f.close()
-            self.credentials = yaml.safe_load(cred)['cred']
+            self._logger.info(f'GS-DBMS Python client version: {self.version}')
+
+            if self.configstr == "":
+                f = open(self.creadPath + "/gscred.yml", "r")
+                cred = f.read()
+                f.close()
+                self.credentials = yaml.safe_load(cred)['cred']
+            else:
+                self.credentials = yaml.safe_load(self.configstr)['cred']
 
             if (('graphqlapiurl' not in self.credentials.keys())
-                    or ('oauthurl' not in self.credentials.keys())
-                    or ('user' not in self.credentials.keys())
-                    or ('password' not in self.credentials.keys())
-                    or ('client_id' not in self.credentials.keys())
-                    or ('client_secret' not in self.credentials.keys())):
+                    or ('token' not in self.credentials.keys())):
                 raise ValueError('error: credentials')
-                return
+            return
         except Exception as e:
-            print(e)
+            self._logger.exception(e)
         return
 
     def getAccessToken(self):
-        if self.accessToken["expires_in_datetime"] < datetime.today():
-            self.refreshToken()
         return self.accessToken["access_token"]
 
     def getTokenFromApi(self):
         try:
-            data = {'grant_type': 'password',
-                    'username': self.credentials['user'],
-                    'password': self.credentials['password']}
+            data = {'username': self.credentials['cnode'],
+                    'password': self.credentials['token']}
             self.getAccessTokenFromApi(data)
         except Exception as e:
-            print(e)
+            self._logger.exception(e)
         return
 
-    def refreshToken(self):
-        data = {'grant_type': 'refresh_token', 'refresh_token': self.accessToken['refresh_token']}
-        self.getAccessTokenFromApi(data)
-
     def getAccessTokenFromApi(self, data):
-        refresh_token_response = requests.post(self.credentials['oauthurl'],
-                                               data=data,
-                                               verify=True,
-                                               allow_redirects=False,
-                                               auth=(
-                                                   self.credentials['client_id'], self.credentials['client_secret']))
-        self.accessToken = json.loads(refresh_token_response.text)
-        self.accessToken["expires_in_datetime"] = datetime.today() + timedelta(
-            seconds=self.accessToken['expires_in'])
+        session = requests.session()
+        refresh_token_response = session.post(
+            self.credentials['baseurl'] + self.credentials['login'],
+            json=data,
+            headers={'Content-Type': 'application/json'})
+        self.accessToken = refresh_token_response.text
+        if refresh_token_response.status_code > 299:
+            self._logger.warn("Error: Login failed with " + str(refresh_token_response.status_code))
+            exit(-1)
+
+        self.cookiejar = session.cookies
+
+
 
         if 'error' in self.accessToken:
-            raise ValueError('error:' + self.accessToken['error'])
+            self._logger.exception("could not authenticate")
 
     def setupCGSResponseCodes(self):
         self.CGSResponseCode['0'] = 'SUCCESS'
@@ -272,7 +273,6 @@ class GSDBS:
         dataSchema = self.getDataSchema()
         if dataSchema == None:
             return
-
         self.statement = \
             f"""mutation {{\n\taddDTable(dtablename: "{self.dtablename}", \n\t\tsuperDTable: [{self.superdtablename}], \n\t\tsriBuildInfo: "{buildInfo}", \n\t\tdataLinks: [\n{dataSchema}\n\t\t], \n\t\tdata: \n{self.getData()}\t\t\n\t)\n}}"""
         return
@@ -318,7 +318,7 @@ class GSDBS:
             dfType = self.getGSDBSType(str(self.data.dtypes[col]))
             linkType = self.getLinkType(schemaLinks, col)
             if dfType.upper() != linkType.upper():
-                print('ERROR: dataframe datatype ', dfType, ' doesnt\' matches dTable datatype', linkType)
+                self._logger.error('ERROR: dataframe datatype ', dfType, ' doesnt\' matches dTable datatype', linkType)
                 return -7
         return 0
 
@@ -338,21 +338,17 @@ class GSDBS:
         if rc != 0: return rc
         rc = self.setData(dataframe)
         if rc != 0: return rc
-
         if schemaCheck == True:
             rc = await self.asyncSchemaCheck()
             if rc != 0: return rc
-
         self.generateMutationStatement()
         if self.debugModus:
-            print(self.statement)
-
+            self._logger.debug(self.statement)
         try:
             rc = await self.asyncExecuteStatement()
         except Exception as e:
-            print(e)
+            self._logger.exception(e)
             return e
-
         return rc
 
     def addDObject(self, dtablename, sribuildinfo, dataframe, superdtablename="DTABLE", schemaCheck=True):
@@ -363,22 +359,58 @@ class GSDBS:
         if rc != 0: return rc
         rc = self.setData(dataframe)
         if rc != 0: return rc
-
         if schemaCheck == True:
             rc = self.schemaCheck()
             if rc != 0: return rc
-
         self.generateMutationStatement()
         if self.debugModus:
-            print(self.statement)
-
+            self._logger.debug(self.statement)
         try:
             rc = self.executeStatement()
         except Exception as e:
-            print(e)
+            self._logger.exception(e)
+            return e
+        return rc
+
+    def addDObjectXSV(self, data):
+        self.statement = self.generateStatement(data)
+        if self.debugModus:
+            self._logger.debug(self.statement)
+        try:
+            rc = self.executeStatement()
+        except Exception as e:
+            self._logger.exception(e)
             return e
 
-        return rc
+    def addDObjectv2XSV(self, data):
+        self.statement = self.generateV2Statement(data)
+        if self.debugModus:
+            self._logger.debug(self.statement)
+        try:
+            rc = self.executeStatement()
+        except Exception as e:
+            self._logger.exception(e)
+            return e
+
+    def updateDObjectXSV(self, data):
+        self.statement = self.generateXsvUpdateStatement(data)
+        if self.debugModus:
+            self._logger.debug(self.statement)
+        try:
+            rc = self.executeStatement()
+        except Exception as e:
+            self._logger.exception(e)
+            return e
+
+    def deleteDDatalinkXSV(self, data):
+        self.statement = self.generateXsvDeleteStatement(data)
+        if self.debugModus:
+            self._logger.debug(self.statement)
+        try:
+            rc = self.executeStatement()
+        except Exception as e:
+            self._logger.exception(e)
+            return e
 
     async def asyncSchemaQuery(self, dtablename):
         statement = "{ __type(name: \"" + dtablename + "\") {name fields { name type { name kind } } } }"
@@ -386,7 +418,7 @@ class GSDBS:
         try:
             result = await self.asyncExecuteStatement(statement)
         except Exception as e:
-            print(e)
+            self._logger.exception(e)
             return e
 
         return result
@@ -397,7 +429,7 @@ class GSDBS:
         try:
             result = self.executeStatement(statement)
         except Exception as e:
-            print(e)
+            self._logger.exception(e)
             return e
 
         return result
@@ -411,6 +443,7 @@ class GSDBS:
 
         try:
             result = await self.prepareClient().execute_async(query)
+
         except Exception as e:
             if "invalid_token" in str(e):
                 self.refreshToken()
@@ -418,10 +451,7 @@ class GSDBS:
             else:
                 return e
 
-        return result
-
     def executeStatement(self, statement=None):
-        global result
         if statement == None:
             if self.statement == None: return -9
             query = gql(self.statement)
@@ -429,19 +459,16 @@ class GSDBS:
             query = gql(statement)
 
         try:
-            result = self.prepareClient().execute(query)
+            return self.prepareClient().execute(query)
+
         except Exception as e:
-            if "invalid_token" in str(result):
-                self.refreshToken()
-                result = self.executeStatement(statement)
-            else:
-                return e
-        return result
+            logging.exception(e)
 
     def prepareClient(self):
-        api_call_headers = {'Authorization': 'Bearer ' + self.getAccessToken()}
+
         return Client(
-            transport=AIOHTTPTransport(url=self.credentials['graphqlapiurl'], headers=api_call_headers),
+            transport=AIOHTTPTransport(url=self.credentials["baseurl"] + self.credentials['graphqlapiurl'],
+                                       cookies=self.cookiejar),
             execute_timeout=40)
 
     async def asyncDropDTable(self, dtablename):
@@ -540,16 +567,91 @@ class GSDBS:
 
     def getFileFromCloud(self, filename):
         filename = filename.replace("/", "$")
-        storageURL = "https://glass-sphere-ai.de/storage/file/" + filename
-        headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
-        resp = requests.get(storageURL, headers=headers)
+        storageURL = self.credentials["baseurl"] + self.credentials['storageurl'] + "/file/" + filename
+        # headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
+        resp = requests.get(storageURL, cookies=self.cookiejar)
         buf = io.BytesIO(resp.content)
         return buf
 
     def saveFileInCloud(self, filenamewithpath, fileasbytesio, fileformat):
-        storageURL = "https://glass-sphere-ai.de/storage/file"
-        headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
+        storageURL = self.credentials["baseurl"] + self.credentials['storageurl'] + "/file"
+        # headers = {'Authorization': 'Bearer ' + self.accessToken['access_token']}
         files = [
             ('filePart', (filenamewithpath, fileasbytesio.getvalue(), fileformat)),
         ]
-        res = requests.post(storageURL, headers=headers, files=files)
+        res = requests.post(storageURL, cookies=self.cookiejar, files=files)
+
+    def generateStatement(self, data):
+        nl = '"'
+        return f"""mutation 
+        {{  addDTable(
+                dtablename: "{data["dtablename"]}",
+                superDTable: [{data["superdtable"].upper()}],
+                sriBuildInfo: "{"-".join(map(lambda x: f"${{{x}}}", data["sribuildinfo"]))}",
+                dataLinks: [{",".join(
+            map(lambda x:
+                f'{{alias:"{x["name"]}", locale: DE, superPropertyURI: DYNAMIC_DATALINK, DataType: {x["type"].upper()}}}'
+                , data["header"]))}],
+                data:
+                [ [{",".join(map(lambda x: f'"{x["name"]}"', data["header"]))}]
+                  {",".join(
+            map(
+                lambda row: f'[{",".join(map(lambda col: f"{nl}{col}{nl}", row))}]', data["rowdata"]))}
+                ]
+                )}}"""
+
+    def generateV2Statement(self, data):
+        nl = '"'
+        empty = ""
+        ignore = "IGNORE"
+        dontignore = "DONTIGNORE"
+        return f"""mutation 
+        {{  addDTablev2(
+                dtablename: "{data["dtablename"]}",
+                superDTable: [{data["superdtable"].upper()}],
+                sriBuildInfo: "{"-".join(map(lambda x: f"${{{x}}}", data["sribuildinfo"]))}",
+                dataLinks: [{",".join(
+            map(lambda x:
+                f'{{alias:"{x["name"]}", locale: DE, superPropertyURI: DYNAMIC_DATALINK, DataType: {x["type"].upper()}}}'
+                , data["header"]))}],
+                data:
+                [ [{",".join(map(lambda x: f'{{value:{nl}{x["name"]}{nl}}}', data["header"]))}]
+                  {",".join(
+            map(
+                lambda row: f'[{",".join(map(lambda col: f"{{value:{nl}{empty if col is None else col}{nl},state:{ignore if col is None else dontignore}}}", row))}]', data["rowdata"]))}
+                ]
+                )}}"""
+
+    def generateXsvUpdateStatement(self, data):
+        stmnt = f'mutation{{' \
+                f'updateDTable(dtablename:{data["dtablename"]}' \
+                f' where: ['
+        for e in data['where']:
+            connective = e['connective']
+            column = e['column']
+            operator = e['operator']
+            value = e['value']
+            stmnt += f'{{connective: {connective}, column: {column}, operator: {operator}, value: "{value}"}}'
+        stmnt += f']' \
+                 f' updatelist: ['
+        for key, value in data['update'].items():
+            stmnt += f'{{datalink: {key}, value: "{value}"}}'
+        stmnt += f'])}}'
+        return stmnt
+
+    def generateXsvDeleteStatement(self, data):
+        stmnt = f'mutation{{' \
+                f'deleteDDatalink(dtablename:{data["dtablename"]}' \
+                f' where: ['
+        for e in data['where']:
+            connective = e['connective']
+            column = e['column']
+            operator = e['operator']
+            value = e['value']
+            stmnt += f'{{connective: {connective}, column: {column}, operator: {operator}, value: "{value}"}}'
+        stmnt += f']' \
+                 f' updatelist: ['
+        for key, value in data['update'].items():
+            stmnt += f'{{datalink: {key}}}'
+        stmnt += f'])}}'
+        return stmnt
